@@ -40,6 +40,7 @@
 
 #include "terminal-intl.h"
 #include "simple-x-font-selector.h"
+#include "terminal.h"
 
 static void egg_xfont_selector_class_init    (EggXFontSelectorClass  *klass);
 static void egg_xfont_selector_init          (EggXFontSelector       *frame);
@@ -330,10 +331,17 @@ egg_xfont_selector_init (EggXFontSelector *selector)
 
   gtk_widget_pop_composite_child ();
 
-  gtk_label_set_mnemonic_widget (GTK_LABEL(selector->family_label),
+  gtk_label_set_mnemonic_widget (GTK_LABEL (selector->family_label),
 				 selector->family_options);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(size_label),
+  terminal_util_set_labelled_by (selector->family_options, GTK_LABEL(selector->family_label));
+  terminal_util_set_atk_name_description (selector->family_options,
+                                          NULL, _("Click to choose font type"));
+  gtk_label_set_mnemonic_widget (GTK_LABEL (size_label),
 				 selector->size_options);
+  terminal_util_set_labelled_by (selector->size_options,
+                                 GTK_LABEL (size_label));
+  terminal_util_set_atk_name_description (selector->size_options,
+                                          NULL, _("Click to choose font size"));
 
   /* FIXME: figure out what spacing looks best */
 
