@@ -27,6 +27,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <libgnome/gnome-program.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <gdk/gdkx.h>
@@ -43,6 +44,8 @@ struct _TerminalWindowPrivate
   GtkWidget *terminal_menuitem;
   GtkWidget *go_menuitem;
   GtkWidget *help_menuitem;
+  GtkWidget *new_window_menuitem;
+  GtkWidget *new_tab_menuitem;
   GtkWidget *copy_menuitem;
   GtkWidget *paste_menuitem;
   GtkWidget *fullscreen_menuitem;
@@ -58,6 +61,7 @@ struct _TerminalWindowPrivate
   GdkPixbuf *icon;
   int old_char_width;
   int old_char_height;
+  void *old_geometry_widget; /* only used for pointer value as it may be freed */
   GConfClient *conf;
   guint notify_id;
   guint menubar_visible : 1;
@@ -309,6 +313,172 @@ fill_in_config_picker_submenu (TerminalWindow *window)
   g_list_free (profiles);  
 }
 
+static void
+fill_in_new_term_submenus (TerminalWindow *window)
+{
+  GtkWidget *new_window_menu;
+  GtkWidget *new_tab_menu;
+  GtkWidget *menu_item;
+  GList *profiles;
+  GList *tmp;
+  GSList *group;
+  GtkAccelGroup *accel_group;
+  char *str;
+  TerminalProfile *default_profile;
+  int i;
+  
+  default_profile = terminal_profile_get_default ();
+
+  if (default_profile == NULL)
+    {
+      gtk_widget_set_sensitive (window->priv->new_window_menuitem, FALSE);
+      gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->new_window_menuitem),
+                                 NULL);
+      gtk_widget_set_sensitive (window->priv->new_tab_menuitem, FALSE);
+      gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->new_tab_menuitem),
+                                 NULL);
+
+      return;      
+    }
+
+  /* FIXME profile name can have an underscore which breaks
+   * the mnemonic stuff
+   */
+  /* avoid "Default (Default)" but have "Default (foobar)" */
+  if (strcmp (terminal_profile_get_visible_name (default_profile),
+              "Default") == 0)
+    str = g_strdup (_("_Default"));
+  else
+    str = g_strdup_printf (_("_Default (%s)"),
+                           terminal_profile_get_visible_name (default_profile));
+  
+  gtk_widget_set_sensitive (window->priv->new_window_menuitem, TRUE);
+  gtk_widget_set_sensitive (window->priv->new_tab_menuitem, TRUE);
+  
+  accel_group = terminal_accels_get_group_for_widget (GTK_WIDGET (window));  
+
+  /* New window submenu */
+  new_window_menu = gtk_menu_new ();
+  gtk_menu_set_accel_group (GTK_MENU (new_window_menu), accel_group);
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->new_window_menuitem),
+                             new_window_menu);
+
+  /* Add default menu item */
+  menu_item = gtk_menu_item_new_with_mnemonic (str);
+  gtk_widget_show (menu_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (new_window_menu),
+                         menu_item);
+  g_signal_connect (G_OBJECT (menu_item),
+                    "activate",
+                    G_CALLBACK (new_window_callback), window);
+  g_object_ref (G_OBJECT (default_profile));
+  g_object_set_data_full (G_OBJECT (menu_item),
+                          "profile",
+                          default_profile,
+                          (GDestroyNotify) g_object_unref);  
+
+  gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
+                                ACCEL_PATH_NEW_WINDOW);
+  
+  /* separator */
+  menu_item = gtk_separator_menu_item_new ();
+  gtk_widget_show (menu_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (new_window_menu),
+                         menu_item);
+  
+  /* New tab submenu */
+  new_tab_menu = gtk_menu_new ();
+  gtk_menu_set_accel_group (GTK_MENU (new_tab_menu), accel_group);
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (window->priv->new_tab_menuitem),
+                             new_tab_menu);
+
+  /* Add default menu item */
+  menu_item = gtk_menu_item_new_with_mnemonic (str);
+  gtk_widget_show (menu_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (new_tab_menu),
+                         menu_item);
+  g_signal_connect (G_OBJECT (menu_item),
+                    "activate",
+                    G_CALLBACK (new_tab_callback), window);
+  g_object_ref (G_OBJECT (default_profile));
+  g_object_set_data_full (G_OBJECT (menu_item),
+                          "profile",
+                          default_profile,
+                          (GDestroyNotify) g_object_unref);
+
+  gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
+                                ACCEL_PATH_NEW_TAB);
+  
+  /* separator */
+  menu_item = gtk_separator_menu_item_new ();
+  gtk_widget_show (menu_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (new_tab_menu),
+                         menu_item);
+  
+  g_free (str);
+
+  i = 1;
+  group = NULL;
+  profiles = terminal_profile_get_list ();
+  tmp = profiles;
+  while (tmp != NULL)
+    {
+      TerminalProfile *profile;
+      
+      profile = tmp->data;
+      
+      /* Profiles can go away while the menu is up. */
+      g_object_ref (G_OBJECT (profile));
+      g_object_ref (G_OBJECT (profile));
+
+      /* FIXME underscores in profile name result in badness */
+      if (i < 10)
+        str = g_strdup_printf (_("_%d. %s"),
+                               i, terminal_profile_get_visible_name (profile));
+      else if (i < 36)
+        str = g_strdup_printf (_("_%c. %s"),
+                               ('A' + i - 10),
+                               terminal_profile_get_visible_name (profile));
+      else
+        str = g_strdup (terminal_profile_get_visible_name (profile));
+      
+      /* item for new window */
+      menu_item = gtk_menu_item_new_with_mnemonic (str);
+      gtk_widget_show (menu_item);
+      gtk_menu_shell_append (GTK_MENU_SHELL (new_window_menu),
+                             menu_item);      
+      g_signal_connect (G_OBJECT (menu_item),
+                        "activate",
+                        G_CALLBACK (new_window_callback),
+                        window);      
+      g_object_set_data_full (G_OBJECT (menu_item),
+                              "profile",
+                              profile,
+                              (GDestroyNotify) g_object_unref);
+
+      /* item for new tab */
+      menu_item = gtk_menu_item_new_with_mnemonic (str);
+      gtk_widget_show (menu_item);
+      gtk_menu_shell_append (GTK_MENU_SHELL (new_tab_menu),
+                             menu_item);      
+      g_signal_connect (G_OBJECT (menu_item),
+                        "activate",
+                        G_CALLBACK (new_tab_callback),
+                        window);      
+      g_object_set_data_full (G_OBJECT (menu_item),
+                              "profile",
+                              profile,
+                              (GDestroyNotify) g_object_unref);
+
+      g_free (str);
+
+      ++i;
+      tmp = tmp->next;
+    }
+
+  g_list_free (profiles);  
+}
+
 static void
 terminal_menu_activated (GtkMenuItem    *menu_item,
                          TerminalWindow *window)
@@ -316,6 +486,13 @@ terminal_menu_activated (GtkMenuItem    *menu_item,
   fill_in_config_picker_submenu (window);
 }
 
+static void
+file_menu_activated (GtkMenuItem    *menu_item,
+                     TerminalWindow *window)
+{
+  fill_in_new_term_submenus (window);
+}
+
 static void
 terminal_window_init (TerminalWindow *window)
 {
@@ -337,7 +514,8 @@ terminal_window_init (TerminalWindow *window)
   
   window->priv->old_char_width = -1;
   window->priv->old_char_height = -1;
-
+  window->priv->old_geometry_widget = NULL;
+  
   window->priv->use_mnemonics = TRUE;
   window->priv->using_mnemonics = FALSE;
   
@@ -363,19 +541,24 @@ terminal_window_init (TerminalWindow *window)
                         "", NULL,
                         NULL, NULL);
   window->priv->file_menuitem = mi;
+
+  /* Set up a callback to demand-create the New Window/New Tab submenus */
+  /* FIXME they need to be created in advance so accels work */
+  g_signal_connect (G_OBJECT (mi), "activate",
+                    G_CALLBACK (file_menu_activated),
+                    window);
+  
   menu = gtk_menu_new ();
   gtk_menu_set_accel_group (GTK_MENU (menu),
                             accel_group);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
 
-  append_menuitem (menu, _("_New Window"), ACCEL_PATH_NEW_WINDOW,
-                   G_CALLBACK (new_window_callback),
-                   window);
-
-  mi = append_menuitem (menu, _("New _Tab"), ACCEL_PATH_NEW_TAB,
-                        G_CALLBACK (new_tab_callback),
-                        window);
+  window->priv->new_window_menuitem =
+    append_menuitem (menu, _("_New Window"), NULL, NULL, NULL);
 
+  window->priv->new_tab_menuitem =
+    append_menuitem (menu, _("New _Tab"), NULL, NULL, NULL);
+  
   /* This is fairly bogus to have here but I don't know
    * where else to put it really
    */
@@ -943,6 +1126,9 @@ terminal_window_set_size (TerminalWindow *window,
   int grid_height;
   int xpad;
   int ypad;
+
+  /* be sure our geometry is up-to-date */
+  terminal_window_update_geometry (window);
   
   widget = terminal_screen_get_widget (screen);
   
@@ -958,7 +1144,13 @@ terminal_window_set_size (TerminalWindow *window,
    */
   gtk_widget_set_size_request (widget, 2000, 2000);
   gtk_widget_size_request (app, &toplevel_request);
-  gtk_widget_get_child_requisition (widget, &widget_request);
+  gtk_widget_size_request (widget, &widget_request);
+
+#ifdef DEBUG_GEOMETRY
+  g_print ("set size: toplevel %dx%d widget %dx%d\n",
+           toplevel_request.width, toplevel_request.height,
+           widget_request.width, widget_request.height);
+#endif
   
   w = toplevel_request.width - widget_request.width;
   h = toplevel_request.height - widget_request.height;
@@ -969,6 +1161,11 @@ terminal_window_set_size (TerminalWindow *window,
   
   w += xpad + char_width * grid_width;
   h += ypad + char_height * grid_height;
+
+#ifdef DEBUG_GEOMETRY
+  g_print ("set size: grid %dx%d setting %dx%d pixels\n",
+           grid_width, grid_height, w, h);
+#endif
   
   if (even_if_mapped)
     gtk_window_resize (GTK_WINDOW (app), w, h);
@@ -1242,7 +1439,8 @@ terminal_window_update_geometry (TerminalWindow *window)
   terminal_widget_get_cell_size (widget, &char_width, &char_height);
   
   if (char_width != window->priv->old_char_width ||
-      char_height != window->priv->old_char_height)
+      char_height != window->priv->old_char_height ||
+      widget != (GtkWidget*) window->priv->old_geometry_widget)
     {
       int xpad, ypad;
       
@@ -1271,9 +1469,26 @@ terminal_window_update_geometry (TerminalWindow *window)
                                      GDK_HINT_MIN_SIZE |
                                      GDK_HINT_BASE_SIZE);
 
+#ifdef DEBUG_GEOMETRY
+      g_print ("hints: base %dx%d min %dx%d inc %d %d\n",
+               hints.base_width,
+               hints.base_height,
+               hints.min_width,
+               hints.min_height,
+               hints.width_inc,
+               hints.height_inc);
+#endif
+      
       window->priv->old_char_width = hints.width_inc;
       window->priv->old_char_height = hints.height_inc;
+      window->priv->old_geometry_widget = widget;
     }
+#ifdef DEBUG_GEOMETRY
+  else
+    {
+      g_print ("hints: increment unchanged, not setting\n");
+    }
+#endif
 }
 
 /*
@@ -1521,43 +1736,44 @@ terminal_window_get_fullscreen (TerminalWindow *window)
  * Callbacks for the menus
  */
 
-static void
-not_implemented (void)
-{
-  GtkWidget *dialog;
-
-  dialog = gtk_message_dialog_new (NULL,
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_ERROR,
-                                   GTK_BUTTONS_CLOSE,
-                                   "Didn't implement this item yet, sorry");
-  g_signal_connect (G_OBJECT (dialog), "response",
-                    G_CALLBACK (gtk_widget_destroy),
-                    NULL);
-
-  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  
-  gtk_widget_show (dialog);
-}
-
 static void
 new_window_callback (GtkWidget      *menuitem,
                      TerminalWindow *window)
 {
-  terminal_app_new_terminal (terminal_app_get (),
-                             terminal_screen_get_profile (window->priv->active_term),
-                             NULL,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL);
+  TerminalProfile *profile;
+  
+  profile = g_object_get_data (G_OBJECT (menuitem),
+                               "profile");
+
+  g_assert (profile);
+
+  if (!terminal_profile_get_forgotten (profile))
+    {
+      terminal_app_new_terminal (terminal_app_get (),
+                                 terminal_screen_get_profile (window->priv->active_term),
+                                 NULL,
+                                 FALSE, FALSE, NULL, NULL, NULL, NULL);
+    }
 }
 
 static void
 new_tab_callback (GtkWidget      *menuitem,
                   TerminalWindow *window)
 {
-  terminal_app_new_terminal (terminal_app_get (),
-                             terminal_screen_get_profile (window->priv->active_term),
-                             window,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL);
+  TerminalProfile *profile;
+  
+  profile = g_object_get_data (G_OBJECT (menuitem),
+                               "profile");
+
+  g_assert (profile);
+
+  if (!terminal_profile_get_forgotten (profile))
+    {
+      terminal_app_new_terminal (terminal_app_get (),
+                                 terminal_screen_get_profile (window->priv->active_term),
+                                 window,
+                                 FALSE, FALSE, NULL, NULL, NULL, NULL);
+    }
 }
 
 static void
@@ -1772,8 +1988,32 @@ static void
 help_callback (GtkWidget      *menuitem,
                TerminalWindow *window)
 {
-  not_implemented ();
+  GError *err;
 
+  err = NULL;  
+  gnome_help_display ("gnome-terminal", NULL, &err);
+
+  if (err != NULL)
+    {
+      GtkWidget *dialog;
+      
+      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_CLOSE,
+                                       _("There was an error displaying help: %s"),
+                                       err->message);
+      
+      g_signal_connect (G_OBJECT (dialog), "response",
+                        G_CALLBACK (gtk_widget_destroy),
+                        NULL);
+      
+      gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+      
+      gtk_widget_show (dialog);
+
+      g_error_free (err);
+    }
 }
 
 static void
