@@ -589,6 +589,19 @@ main (int argc, char **argv)
     { NULL, NULL }
   };
   GnomeClient *sm_client;
+
+  if (setlocale (LC_ALL, "") == NULL)
+    g_printerr ("GNOME Terminal: locale not understood by C library, internationalization will not work\n");
+
+#if 0
+  {
+    const char *charset;
+    g_get_charset (&charset);
+    
+    g_print ("Running in locale \"%s\" with encoding \"%s\"\n",
+             setlocale (LC_ALL, NULL), charset);
+  }
+#endif
   
   bindtextdomain (GETTEXT_PACKAGE, TERM_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -1586,10 +1599,13 @@ terminal_app_new_profile (TerminalApp     *app,
       gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
       entry = gtk_entry_new ();
       app->new_profile_name_entry = entry;
+      terminal_util_set_atk_name_description (entry, _("Enter profile name"), NULL);
+
 
       gtk_entry_set_width_chars (GTK_ENTRY (entry), 14);
       gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+      terminal_util_set_labelled_by (entry, GTK_LABEL (label));
       
       gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
@@ -1607,6 +1623,7 @@ terminal_app_new_profile (TerminalApp     *app,
                                          default_base_profile);
       
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), option_menu);
+      terminal_util_set_labelled_by (option_menu, GTK_LABEL (label));
       
       gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), option_menu, FALSE, FALSE, 0);
@@ -1745,6 +1762,7 @@ create_profile_list (void)
                               G_TYPE_OBJECT);
   
   tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+  terminal_util_set_atk_name_description (tree_view, _("Profile list"), NULL);
 
   g_object_unref (G_OBJECT (model));
   
@@ -2085,6 +2103,8 @@ terminal_app_manage_profiles (TerminalApp     *app,
                                      GTK_STOCK_CLOSE,
                                      GTK_RESPONSE_ACCEPT,
                                      NULL);
+      gtk_dialog_set_default_response (GTK_DIALOG (app->manage_profiles_dialog),
+                                       GTK_RESPONSE_ACCEPT);
       g_signal_connect (G_OBJECT (app->manage_profiles_dialog),
                         "response",
                         G_CALLBACK (gtk_widget_destroy),
@@ -2113,7 +2133,9 @@ terminal_app_manage_profiles (TerminalApp     *app,
       label = gtk_label_new_with_mnemonic (_("Profile _used when launching a new terminal:"));
       gtk_label_set_mnemonic_widget (GTK_LABEL (label),
                                      app->manage_profiles_default_menu);
-      
+      terminal_util_set_labelled_by (GTK_WIDGET(app->manage_profiles_default_menu),                    
+                                     GTK_LABEL (label));
+
       gtk_box_pack_start (GTK_BOX (hbox),
                           label, TRUE, TRUE, 0);
             
@@ -2143,6 +2165,9 @@ terminal_app_manage_profiles (TerminalApp     *app,
                         app);
       
       sw = gtk_scrolled_window_new (NULL, NULL);
+      terminal_util_set_labelled_by (GTK_WIDGET(app->manage_profiles_list),                            
+                                     GTK_LABEL (label));
+      
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                       GTK_POLICY_AUTOMATIC,
                                       GTK_POLICY_AUTOMATIC);
@@ -2175,6 +2200,9 @@ terminal_app_manage_profiles (TerminalApp     *app,
       g_signal_connect (G_OBJECT (button), "clicked",
                         G_CALLBACK (new_button_clicked), app);
       app->manage_profiles_new_button = button;
+      terminal_util_set_atk_name_description (app->manage_profiles_new_button, NULL,                             
+                                              _("Click to open new profile dialog"));
+
       
       button = gtk_button_new_with_mnemonic (_("_Edit..."));
       fix_button_align (button);
@@ -2183,6 +2211,8 @@ terminal_app_manage_profiles (TerminalApp     *app,
       g_signal_connect (G_OBJECT (button), "clicked",
                         G_CALLBACK (edit_button_clicked), app);
       app->manage_profiles_edit_button = button;
+      terminal_util_set_atk_name_description (app->manage_profiles_edit_button, NULL,                            
+                                              _("Click to open edit profile dialog"));
       
       button = gtk_button_new_with_mnemonic (_("_Delete..."));
       fix_button_align (button);
@@ -2191,6 +2221,9 @@ terminal_app_manage_profiles (TerminalApp     *app,
       g_signal_connect (G_OBJECT (button), "clicked",
                         G_CALLBACK (delete_button_clicked), app);
       app->manage_profiles_delete_button = button;
+      terminal_util_set_atk_name_description (app->manage_profiles_delete_button, NULL,                          
+                                              _("Click to delete selected profile"));
+
       
       /* Set default size of profile list */
       gtk_window_set_geometry_hints (GTK_WINDOW (app->manage_profiles_dialog),
@@ -2238,6 +2271,7 @@ profile_optionmenu_new (void)
   GtkWidget *option_menu;
   
   option_menu = gtk_option_menu_new ();
+  terminal_util_set_atk_name_description (option_menu, NULL, _("Click button to choose profile"));
 
   profile_optionmenu_refill (option_menu);
   
@@ -2512,3 +2546,73 @@ save_yourself_callback (GnomeClient        *client,
   /* success */
   return TRUE;
 }
+
+
+/*
+ * Utility stuff
+ */
+
+
+
+/* This function is used to set LABLLED_BY relation between widgets
+ * and labels
+ */
+void
+terminal_util_set_labelled_by (GtkWidget *widget,
+                               GtkLabel  *label)
+{
+  AtkObject *atkwidget;
+  AtkRelationSet *relation_set;
+  AtkRelation *relation;
+  AtkObject *targets[1];
+
+  atkwidget = gtk_widget_get_accessible (widget);
+
+  if (atkwidget == NULL)
+    {
+      g_warning ("%s: for some reason widget has no GtkAccessible",
+                 G_GNUC_FUNCTION);
+      return;
+    }
+
+  if (!GTK_IS_ACCESSIBLE (atkwidget))
+    return; /* This means GAIL is not loaded so we have the NoOp accessible */
+  
+  relation_set = atk_object_ref_relation_set (atkwidget);
+  targets[0] = gtk_widget_get_accessible (GTK_WIDGET (label));
+
+  relation = atk_relation_new (targets, 1, ATK_RELATION_LABELLED_BY);
+  atk_relation_set_add (relation_set, relation);
+  g_object_unref (G_OBJECT (relation));
+
+}
+
+/* sets accessible name and description for the widget */
+
+void
+terminal_util_set_atk_name_description (GtkWidget  *widget,
+                                        const char *name,
+                                        const char *desc)
+{
+  AtkObject *obj;
+  
+  obj = gtk_widget_get_accessible (widget);
+
+  if (obj == NULL)
+    {
+      g_warning ("%s: for some reason widget has no GtkAccessible",
+                 G_GNUC_FUNCTION);
+      return;
+    }
+
+  
+  if (!GTK_IS_ACCESSIBLE (obj))
+    return; /* This means GAIL is not loaded so we have the NoOp accessible */
+      
+  g_return_if_fail (GTK_IS_ACCESSIBLE (obj));  
+  if (desc)
+    atk_object_set_description (obj, desc);
+  if (name)
+    atk_object_set_name (obj, name);
+}
+
